#include <math.h>
#include <stdlib.h>
#include "imgtools.h"
#include "my_matrix_stuff.h"

void threshold(unsigned char** inputImage, unsigned char** outputImage, unsigned char thresh, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         outputImage[i][j]=((inputImage[i][j])>thresh);
      }
   }
}

void multiplyImage(unsigned char** inputImage, unsigned char** outputImage, int factor, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {  
         outputImage[i][j]=(inputImage[i][j])*factor;
      }
   }
}

void notImage(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j]=!(inputImage[i][j]);
      }
   }
}

void orImage(unsigned char** inputImage1, unsigned char** inputImage2, unsigned char** outputImage, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j]=(inputImage1[i][j])||(inputImage2[i][j]);
      }
   }
}

void clearImage(unsigned char** image, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
        image[i][j]=0;
      }
   }
}

void relabel(unsigned char** inputImage, unsigned char** outputImage, int oldClass, int newClass, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if((inputImage[i][j])==oldClass)
            outputImage[i][j]=newClass;
      }
   }
}

void skiplabel(unsigned char** mask, int label2, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if((mask[i][j])>label2) { (mask[i][j])--; }
      }
   }
}

int max(unsigned char** image, int rows, int cols)
{
   int i,j;
   unsigned char currentMax = image[0][0];

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         if(image[i][j]>currentMax) {
            currentMax=image[i][j];
         }
      }
   }

   return currentMax;
}

void find_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j] = (inputImage[i][j]==valueToFind);
      }
   }
}

void find_two_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind1, unsigned char valueToFind2, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j] = (inputImage[i][j]==valueToFind1) ||
                             (inputImage[i][j]==valueToFind2);
      }
   }
}

void ifthen(unsigned char** ifImage, unsigned char** thenImage, unsigned char** outputImage, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         outputImage[i][j] = thenImage[i][j]*((ifImage[i][j])>0);
      }
   }
}

void findHoles(unsigned char** inputImage, unsigned char** holes, int lowX, int highX, int lowY, int highY)
{
   int i;
   unsigned char** temp=allocateByteMatrix(highY+1, highX+1);

   notImage(inputImage, temp, lowX, highX, lowY, highY);
   for(i = lowX; i<=highX; i++) {
      temp[lowY][i]=1;
      temp[highY][i]=1;
   }
   for(i = lowY; i<=highY; i++) {
      temp[i][lowX]=1;
      temp[i][highX]=1;
   }
   labelClusters(temp, holes, lowX, highX, lowY, highY, 8);
   find_equal(holes, holes, 1, lowX, highX, lowY, highY);
   orImage(holes, inputImage, holes, lowX, highX, lowY, highY);
   notImage(holes, holes, lowX, highX, lowY, highY);

   freeByteMatrix(temp);
}

void labelSingleObject(unsigned char** inputImage, unsigned char** outputImage,int startX, int startY, unsigned char class, int lowX, int highX, int lowY, int highY, int connectedness)
{
   const int connectedX[] = X_NEIGHBORS;
   const int connectedY[] = Y_NEIGHBORS;
   int i, pixelQueueSize=0;
   int x, y;
   int pixelQueue[(1+highY-lowY)*(1+highX-lowX)];

   outputImage[startY][startX]=class;
   pixelQueue[0] = (startY << 16) + startX;
   pixelQueueSize = 1;

   while(pixelQueueSize > 0) {
      x=pixelQueue[0] & 0xffff;
      y=(pixelQueue[0] >> 16) & 0xffff;
      pixelQueue[0]=pixelQueue[--pixelQueueSize];
      for(i = 0; i < connectedness; i++) {
         if (((x+connectedX[i])>=lowX)&&((x+connectedX[i])<=highX)&&
             ((y+connectedY[i])>=lowY)&&((y+connectedY[i])<=highY)&&
             (inputImage[y+connectedY[i]][x+connectedX[i]]>0)&&
             (outputImage[y+connectedY[i]][x+connectedX[i]]!=class)) {
            outputImage[y+connectedY[i]][x+connectedX[i]]=class;
            pixelQueue[pixelQueueSize++] = ((y+connectedY[i]) << 16) + (x+connectedX[i]);
         }
      }
   }
}

int labelClusters(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY, int connectedness)
{
   int i,j;
   unsigned char currentClass=1;

   clearImage(outputImage, lowX, highX, lowY, highY);
   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if((inputImage[i][j]>0)&&(outputImage[i][j]<1)) {
            outputImage[i][j]=currentClass;
            labelSingleObject(inputImage, outputImage, j, i, currentClass, lowX,
                              highX, lowY, highY, connectedness);
            currentClass++;
         }
      }
   }
   return(currentClass-1);
}

void findBoundary(unsigned char** inputImage, unsigned char** outputImage, unsigned char** holes, int rows, int cols)
{
   int i,j,k;
   int isBoundary;
   const int connectedX[] = X_NEIGHBORS;
   const int connectedY[] = Y_NEIGHBORS;
   const int CONNECTEDNESS = 4;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         isBoundary=0;
         for(k = 0; k < CONNECTEDNESS; k++) {
            if (((j+connectedX[k])>=0)&&((j+connectedX[k])<cols)&&
                ((i+connectedY[k])>=0)&&((i+connectedY[k])<rows)&&
                (inputImage[i][j]>0)&&
                ((inputImage[i+connectedY[k]][j+connectedX[k]])<1)&&
		(!holes[i+connectedY[k]][j+connectedX[k]])) {
               isBoundary=1;
            }
         }
         outputImage[i][j]=isBoundary;
      }
   }
}
