/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)imgtools.h    1.0     02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@.ece.utexas.edu.
*/


/* Constants */
#ifndef MAXFLOAT
#define MAXFLOAT 100000
#endif

// Set of the coordinates for all 8 connected neigbors starting with
//    the four connected neighbors
#define X_NEIGHBORS {0, -1, 1, 0, -1, 1, -1, 1}
#define Y_NEIGHBORS {1, 0, 0, -1, -1, -1, 1, 1}

/* Various image processing functions */

// If inputImage pixels are above thresh, set corresponding pixel in
//    outputImage to 1
void threshold(unsigned char** inputImage, unsigned char** outputImage, unsigned char thresh, int rows, int cols);

// outputImage = inputImage * factor
void multiplyImage(unsigned char** inputImage, unsigned char** outputImage, int factor, int rows, int cols);

// outputImage = ! inputImage
void notImage(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY);

// outputImage = inputImage1 || inputImage2
void orImage(unsigned char** inputImage1, unsigned char** inputImage2, unsigned char** outputImage, int lowX, int highX, int lowY, int highY);

// Set section of image to zeros
void clearImage(unsigned char** image, int lowX, int highX, int lowY, int highY);

// Change all pixels of value oldClass in inputImage to newClass in outputImage
void relabel(unsigned char** inputImage, unsigned char** outputImage, int oldClass, int newClass, int lowX, int highX, int lowY, int highY);

// Decrement all pixel values above label2
void skiplabel(unsigned char** mask, int label2, int lowX, int highX, int lowY, int highY);

// Find the maximum value in an image
int max(unsigned char** image, int rows, int cols);

// outputImage = (inputImage == valueToFind)
void find_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind, int lowX, int highX, int lowY, int highY);

// outputImage = (inputImage == valueToFind1) || (inputImage == valueToFind2)
void find_two_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind1, unsigned char valueToFind2, int lowX, int highX, int lowY, int highY);

// outputImage = (ifImage > 0) * thenImage
void ifthen(unsigned char** ifImage, unsigned char** thenImage, unsigned char** outputImage, int rows, int cols);

// Find holes in non-zero objects in inputImage and return image of holes
//    where positions of holes will have pixel values of 1
void findHoles(unsigned char** inputImage, unsigned char** holes, int lowX, int highX, int lowY, int highY);

// Given seed position (startX, startY), label all connecting pixels as class
void labelSingleObject(unsigned char** inputImage, unsigned char** outputImage,int startX, int startY, unsigned char class, int lowX, int highX, int lowY, int highY, int connectedness);

// Label all connected components in inputImage and return number of objects
//    found.
int labelClusters(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY, int connectedness);

// Set boundary of objects in inputImage to 1 in outputImage
void findBoundary(unsigned char** inputImage, unsigned char** outputImage, unsigned char** holes, int rows, int cols);
