/*
Copyright (c) 1998 The University of Texas
All Rights Reserved.
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.
 
Programmers:	Wade Schwartzkopf
Version:        @(#)mf_segment.c	.1	02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@ece.utexas.edu.
*/

/*
An example of running this program is:

mfSegment ClassifiedMap segmented 517 645

*/

/* Includes */

#include <stdio.h>
#include "readIntArg.h"
#include "my_matrix_stuff.h"
#include "segmentation.h"

/* Main routine */
int main(int argc, char *argv[])
{
   const int minArgs=2;
   const int maxArgs=4;
   int imageType = 0;
   int numRows = 0; // 517
   int numColumns = 0; // 645
   unsigned char **inputByteImage = NULL, **outputByteImage = NULL;

   /* Check for valid arguments */
   if (argc < (minArgs+1)) {
      fprintf(stderr, "You passed %d arguments and at least %d arguments are required.\n", argc - 1, minArgs);
      exit(1);
   }
   else if (argc > maxArgs+1) {
      fprintf(stderr, "You passed too many arguments.  Ignoring extra arguments.\n");
   }

   if (argc == 4) {
      numColumns = (numRows = readIntArg("Number of rows", argv[3], 1));
   }
   else if (argc >= 5) {
      numRows = readIntArg("Number of rows", argv[3], 1);
      numColumns = readIntArg("Number of columns", argv[4], 1);
   }

   /* Read image in 2-D array of bytes */
   // If PGM or PNG format, set numRows and numColumns from header data.
   imageType = readHeader(argv[1], &numRows, &numColumns);
   if(imageType<0) {
      fprintf(stderr,
              "File '%s' could not be opened", argv[1]);
      exit(1);
   }
   if((numRows<=0) || (numColumns<=0)) {
      fprintf(stderr,
              "File '%s' is not PGM or PNG format.  "
              "Reading raw image data requires valid arguments "
              "for image height and width.\n", argv[1]);
      exit(1);
   }
   inputByteImage = allocateByteMatrix(numRows, numColumns);
   readImageAs2DByte(argv[1], inputByteImage, &numRows, &numColumns);
 
   /* Segment image */
   outputByteImage = allocateByteMatrix(numRows, numColumns);
   segment(inputByteImage, outputByteImage, numRows, numColumns);

   /* Save output image */
   writeImageAs2DByte(argv[2], outputByteImage, numRows, numColumns, imageType);

   freeByteMatrix(inputByteImage);
   freeByteMatrix(outputByteImage);

   return 0;
}
