/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)my_matrix_stuff.c    1.0     02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@.ece.utexas.edu.
*/

#include <math.h>
#include <stdlib.h>
#include "image_io.h"
#include "my_matrix_stuff.h"

/*  Allocate a 2D byte matrix */
unsigned char** allocateByteMatrix(int height, int width)
{
    int i;
    unsigned char** byteMatrix = (unsigned char **) malloc(height*sizeof(unsigned char *));
 
    if (!byteMatrix) {
        fprintf(stderr, "Cannot allocate memory in allocateByteMatrix.\n");
        exit(1);
    }
    byteMatrix[0] = (unsigned char *) malloc(width*height*sizeof(unsigned char));
    if (!byteMatrix[0]) {
        fprintf(stderr, "Cannot allocate matrix in allocateByteMatrix.\n");
        exit(2);
    }
    for (i = 1; i < height; i++) byteMatrix[i] = byteMatrix[i-1] + width;
    return(byteMatrix);
}

/*  Deallocate a 2D byte matrix */
int freeByteMatrix(unsigned char **byteMatrix)
{
    if (byteMatrix == 0) {
        return 0;
    }
    if (byteMatrix[0]) {
        free(byteMatrix[0]);
        byteMatrix[0] = 0;
    }
    free(byteMatrix);
    return 1;
}

void copy_image(unsigned char** inputImage, unsigned char** outputImage, int rows, int cols)
{
   memcpy(*outputImage, *inputImage, rows*cols);
}

void copy_square(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j]=inputImage[i][j];
      }
   }
}

void byteToTwoD(unsigned char* ByteImage, unsigned char** byte2DImage, int rows, int cols)
{
   int i,j;
   unsigned char* tempBytePtr = ByteImage;

   for (i = 0; i < rows; i++) {
      unsigned char *byteRowImagePtr = byte2DImage[i];
      for (j = 0; j < cols; j++) {
         *byteRowImagePtr++ = *tempBytePtr++;
      }
   }
}

void twoDToByte(unsigned char** byte2DImage, unsigned char* ByteImage, int rows, int cols)
{
   int i,j;
   unsigned char* outputPtr = ByteImage;

   for (i = 0; i < rows; i++) {
      for (j = 0; j < cols; j++) {
         *outputPtr++ = byte2DImage[i][j];
      }
   }
}

int readImageAs2DByte(char* imageFile, unsigned char** byte2DImage, int* rows, int* cols)
{
   int imageType;
   unsigned char *ByteImage = NULL;

   // readByteImage allocates memory for ByteImage
   imageType = readByteImage(imageFile, &ByteImage, rows, cols);
   byteToTwoD(ByteImage, byte2DImage, *rows, *cols);

   free(ByteImage);

   return imageType;
}

void writeImageAs2DByte(char* imageFile, unsigned char** byte2DImage, int rows, int cols, int imageType)
{
   unsigned char *ByteImage = NULL;

   ByteImage = allocateImage(rows, cols, 1);
   twoDToByte(byte2DImage, ByteImage, rows, cols);
   writeByteImage(imageFile, ByteImage, &rows, &cols, imageType);

   free(ByteImage);
}
