/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)segmentation.h    1.0     02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@.ece.utexas.edu.
*/

#include "points.h"
#include "labels.h"

#define INCLUDE_LINE  // Include the dividing line in the entropy calculation
#define PP_ENTROPY  // Return average of entropies instead of sum
#define THRESH_ENTROPY 0.3 // Entropy above which an object is called a cluster
#define THRESH_SIZE 10  // Smallest valid chromosome size (in pixels)
#define THRESH_E_STEP 0.01 // Don't bother with entropy decreases below this size

/* Lots of various functions used in segmenting an MFISH image */

// Check to see if cut line is valid (Is it contained within the object?)
int validCutLine(struct point start, struct point end, unsigned char** classImage, unsigned char** single, unsigned char** holes, int rows, int cols, int lineValue, double* lineArea, double lineHist[]);

// Calculate entropy of object
// If split>0, calculate entropy of two objects
double calculateEntropy(unsigned char** inputImage, unsigned char** mask, int lowX, int highX, int lowY, int highY, int split, double lineArea, double lineHist[], double threshEntropy);

// Select the cut line that results in the lowest entropy
struct pointList* selectCutLine(struct pointList* boundary_points, unsigned char** inputImage, unsigned char** single, unsigned char** labels, unsigned char** holes, int* nextClass, int rows, int cols);

// Determine if object is cluster of multiple chromosomes
int isCluster(unsigned char** inputImage, unsigned char** labels, unsigned char** single, int objectLabel, int lowX, int highX, int lowY, int highY);

// Remove points from list to reduce calculation
void filterBoundaryPoints(unsigned char** inputImage, struct pointList* boundary, int rows, int cols);

// Label two objects as the same object, if they are determined to be the same
void equivalent(unsigned char** inputImage, unsigned char** mask, struct labelList* labelsToMerge, int lowX, int highX, int lowY, int highY);

// Segment chromosome image using image of classified pixels (inputImage)
// Return image of labelled objects (outputImage)
void segment(unsigned char** inputImage, unsigned char** outputImage, int rows, int cols);
